require 'game/world/World'
require 'game/UI'

Game = class('Game')

function Game:init()
    self.n_babies = 0
    love.physics.setMeter(32)
    self.world = love.physics.newWorld(0, 0)
    self.current_world = World(self)
    self.ui = UI()
    self.crosshair_angle = 0
    self.tutorial = false
    self.lost = false 
    self.won = false 
    self.paused = false
    timer:after(1, function() self.tutorial = true; love.mouse.setGrab(false) end)
end

function Game:update(dt)
    if self.paused then return end
    if self.tutorial then return end
    if self.n_babies <= 0 then self.won = true end
    if self.current_world then 
        if self.current_world.player then
            if self.current_world.lost then 
                self.lost = true
                love.mouse.setGrab(false)
                return 
            end
        end
        if self.won then 
            love.mouse.setGrab(false)
            return 
        end
        local v = math.max(math.abs(self.current_world.camera_v.x), 
                           math.abs(self.current_world.camera_v.y))
        self.crosshair_angle = self.crosshair_angle + v*dt*0.1
        self.current_world:update(dt) 
        self.ui:update(dt)
    end    
end

function Game:draw()
    if self.current_world then 
        self.current_world:draw() 
        self.ui:draw()
        love.graphics.setFont(tutorial_text)
        outlineText("Children left: " .. tostring(self.n_babies), 10, 10, {255, 255, 255}, 255)
    end
    if self.paused then self.ui:drawPaused()
    elseif self.tutorial then self.ui:drawTutorial() 
    elseif self.lost then self.ui:drawLost() 
    elseif self.won then self.ui:drawWon() end
    local x, y = love.mouse.getPosition()
    pushRotate(x, y, self.crosshair_angle)
    love.graphics.draw(crosshair, x - 16, y - 16)
    love.graphics.pop()
end

function Game:mousepressed(x, y, button)
    if self.current_world then self.current_world:mousepressed(x, y, button) end
end

function Game:mousereleased(x, y, button)
    if self.current_world then self.current_world:mousepressed(x, y, button) end
end

function Game:keypressed(key)
    if self.current_world then self.current_world:keypressed(key) end
    if key == 'escape' then 
        if self.lost or self.won or self.tutorial then return end
        self.paused = not self.paused 
        if self.paused then
            love.mouse.setGrab(false)
        else love.mouse.setGrab(true) end
    end
    if key == ' ' then 
        if self.tutorial then self.tutorial = false; love.mouse.setGrab(true)
        elseif game.lost then
            self.current_world:destroy()
            self.n_babies = 0
            self.current_world = World(self)
            self.tutorial = false
            self.lost = false
            timer:after(0.2, function() self.tutorial = true end)
        elseif game.won then
            self.current_world:destroy()
            self.n_babies = 0
            self.current_world = World(self)
            self.tutorial = false
            self.won = false
            timer:after(0.2, function() self.tutorial = true end)
        end
    end
end

function Game:keyreleased(key)
    if self.current_world then self.current_world:keyreleased(key) end
end
