local newImage = love.graphics.newImage
local newImageData = love.image.newImageData

crosshair = newImage('resources/crosshair.png')
projectile_b = newImage('resources/projectile_b.png')
projectile_s = newImage('resources/projectile_s.png')
grass = newImage('resources/grass.png')
square = newImage('resources/square.png')
main_pso = 'resources/Zytokine.pso'
aang_left = newImage('resources/aang_left.png')
aang_right = newImage('resources/aang_right.png')
skill_ui = newImage('resources/skill_ui.png')
skill_overlay = newImage('resources/skill_overlay.png')
skill_overlay_yes = newImage('resources/skill_overlay_yes.png')
skill_overlay_no = newImage('resources/skill_overlay_no.png')
skill_overlay_mana = newImage('resources/skill_overlay_mana.png')
bar = newImage('resources/bar.png')
font_damage_pop_small = love.graphics.newFont('resources/visitor1.ttf', 24)
font_damage_pop_medium = love.graphics.newFont('resources/visitor1.ttf', 30)
font_damage_pop_big = love.graphics.newFont('resources/visitor1.ttf', 36)
tutorial_text = love.graphics.newFont('resources/visitor1.ttf', 18)

visuals = {}
visuals['aang_walk_left'] = newImage('resources/aang_walk_left.png')
visuals['aang_walk_right'] = newImage('resources/aang_walk_right.png')
visuals['aang_attack_left'] = newImage('resources/aang_attack_left.png')
visuals['aang_attack_right'] = newImage('resources/aang_attack_right.png')
visuals['baby_walk_left'] = newImage('resources/baby_walk_left.png')
visuals['baby_walk_right'] = newImage('resources/baby_walk_right.png')
visuals['rogue_walk_left'] = newImage('resources/rogue_walk_left.png')
visuals['rogue_walk_right'] = newImage('resources/rogue_walk_right.png')
visuals['rogue_attack_left'] = newImage('resources/rogue_attack_left.png')
visuals['rogue_attack_right'] = newImage('resources/rogue_attack_right.png')
visuals['monk_walk_left'] = newImage('resources/monk_walk_left.png')
visuals['monk_walk_right'] = newImage('resources/monk_walk_right.png')
visuals['monk_attack_left'] = newImage('resources/monk_attack_left.png')
visuals['monk_attack_right'] = newImage('resources/monk_attack_right.png')
visuals['ent_walk_left'] = newImage('resources/ent_walk_left.png')
visuals['ent_walk_right'] = newImage('resources/ent_walk_right.png')
visuals['ent_attack_left'] = newImage('resources/ent_attack_left.png')
visuals['ent_attack_right'] = newImage('resources/ent_attack_right.png')
visuals['entie_walk'] = newImage('resources/entie_walk.png')
visuals['items'] = {}
visuals['items']['mana regen up'] = newImage('resources/mana_regen_up.png')
visuals['items']['hp regen up'] = newImage('resources/hp_regen_up.png')
visuals['items']['mov speed up'] = newImage('resources/mov_speed_up.png')
visuals['items']['attack speed up'] = newImage('resources/attack_speed_up.png')
visuals['items']['mana cost down'] = newImage('resources/mana_cost_down.png')
visuals['items']['skill cooldown down'] = newImage('resources/skill_cooldown_down.png')
visuals['items']['damage up'] = newImage('resources/damage_up.png')
visuals['items']['range up'] = newImage('resources/range_up.png')
item_types = {'mana regen up', 'hp regen up', 'mov speed up', 'attack speed up', 
              'mana cost down', 'damage up', 'range up'}

-- Tiles
tiles_data = {}
-- Returns a table of images from tile sheet = name, with tile width = tile_w and height = tile_h.
-- Reads left->right, top->down: first image: x = 1, y = 1; 
-- wth image: x = tile_w, y = 1, last image: x = tile_w, y = tile_h.
local function getImagesFromTileSheet(name, tile_w, tile_h)
    local tiles = {}
    local data = newImageData('resources/' .. name .. '.png')
    local n_w, n_h = data:getWidth()/tile_w, data:getHeight()/tile_h
    for j = 1, n_h do
        for i = 1, n_w do
            local tile = newImageData(tile_w, tile_h)
            local m, n = 0, 0
            for k = tile_w*(i-1), tile_w*i-1 do
                n = 0
                for l = tile_h*(j-1), tile_h*j-1 do
                    local r, g, b, a = data:getPixel(k, l)
                    tile:setPixel(m, n, r, g, b, a)
                    n = n + 1
                end
                m = m + 1
            end
            table.insert(tiles, newImage(tile))
        end
    end
    return tiles
end
-- Set tile data
tiles_data['environment_32'] = getImagesFromTileSheet('environment_32', 32, 32)
tiles_data['props_trees'] = getImagesFromTileSheet('props_trees', 64, 64)
