Baby = class('Baby', Entity)
Baby:include(PhysicsRectangle)
Baby:include(Timer)
Baby:include(HittableRed)
Baby:include(Steerable)
Baby:include(Speaker)

function Baby:init(world, x, y, settings)
    Entity.init(self, world, x, y, settings)
    self:physicsRectangleInit(self.world.world, self.x, self.y, 'dynamic', 26, 18)
    self:timerInit()
    self:hittableInit(0.2)
    self:steerableInit(0, 0, 50, 1000)
    self:speakerInit()

    self.offset = Vector(-2, -12)
    self.animations = {}
    self.animations['walk_left'] = newAnimation(visuals['baby_walk_left'], 32, 32, 0.2, 0)
    self.animations['walk_right'] = newAnimation(visuals['baby_walk_right'], 32, 32, 0.2, 0)

    self.timer:every(math.prandom(1, 3), function() self:act() end)
    self.current_behavior = 'stop'

    self.hp = 5
    self.particle_colors = {{172, 52, 52}, {200, 64, 64}, {220, 92, 92}}

    self.fear_texts = {
        "no pls", "noooo", "heelpp ;_;", "im too young to die!",
        "go away!!!", "dont kill me pls", "i dont wanna die! ;_;",
        "mooommm", "pls heeelp!!", "send help!!!", "aaaaaaaa",
        "whyyyy :(", "stahhp", "yolobama~~", "im being molested!!!111!",
        "PERVERT!!!", "leave me alone!11", "no!! meanie :(", "fuk u",
        "go away turbonerdd!!", "i'll end youuuu!!"
    }

    self.hurt_texts = {
        "ouchhh", "stahhp", "whyyyyyy", "fuk u", "noooo", "nooooooo ;_;",
        "stop it you turbonerddddd", "stopppp", "pls no", ";_;", "x_;", ";_x",
        "x_x", "croissant!!!!!!", "wawawawa", "runnnnnn", "oh noo", "fgt",
        "im too young to die!", "dont kill me pls", "helppp ;_;", "aaaaaaaa",
        "send helppp!!!", "whyyyy :(", "no pls no", "ill rek u nub"
    }
end

function Baby:update(dt)
    self.x, self.y = self.body:getPosition()
    self:timerUpdate(dt)
    self:steerableUpdate(dt)

    if self.moving then
        if self.direction == 'left' then
            self.animations['walk_left']:update(dt)
        else self.animations['walk_right']:update(dt) end
    else
        if self.direction == 'left' then
            self.animations['walk_left']:seek(1)
        else self.animations['walk_right']:seek(1) end
    end

    local player_x, player_y = self.world.player.x, self.world.player.y
    if Vector.distance(Vector(player_x, player_y), Vector(self.x, self.y)) < 100 then
        if self.current_behavior ~= 'push' then
            self.current_behavior = 'flee'
            self.target = Vector(player_x, player_y)
            self:speak(table.random(self.fear_texts))
        end
    end
end

function Baby:collisionSolid(solid, nx, ny)
    self.v.x = -self.v.x
    self.v.y = -self.v.y
end

function Baby:projectileHit(r)
    self.timer:after(0.5, self:speak(table.random(self.hurt_texts)))
    self:hittableHit()
    self:steerablePush(300, r)
    local color = table.random(self.particle_colors)
    local c1, c2 = {color[1], color[2], color[3], 255}, {color[1], color[2], color[3], 0}
    self.world:particleCreate('ProjHit', {position = {x = self.x, y = self.y},
                              colors = {c1[1], c1[2], c1[3], c1[4], c2[1], c2[2], c2[3], c2[4]}})
    self.hp = self.hp - 1
    self.world:createToGroup('NumberPopText', 'NumberPopText', self.x, self.y,
                            {size = 'small', text = "1", color = {240, 160, 160}})
    if self.hp <= 0 then
        self.world:hitFrameStopAdd(7)
        self.timer:after(0.02, function() 
            self.dead = true 
            game.n_babies = game.n_babies - 1
        end)
    end
end

function Baby:act()
    if self.current_behavior ~= 'flee' and self.current_behavior ~= 'push' then
        local r = math.prandom(0, 2*math.pi)
        local d = math.prandom(25, 75)
        self.current_behavior = 'arrival'
        self.target = Vector(self.x + d*math.cos(r), self.y + d*math.sin(r))
    end
end

function Baby:draw()
    if debug_draw then self:physicsRectangleDraw() end
    self:hittableDraw()

    local x, y = self.body:getPosition()
    x, y = x - self.w/2 + self.offset.x, y - self.h/2 + self.offset.y
    if self.direction == 'left' then
        self.animations['walk_left']:draw(x, y)
    else self.animations['walk_right']:draw(x, y) end

    love.graphics.setColorMode('modulate')
    love.graphics.setColor(255, 255, 255)
end
