Ent = class('Ent', Entity)
Ent:include(PhysicsRectangle)
Ent:include(Timer)
Ent:include(HittableRed)
Ent:include(Steerable)
Ent:include(Speaker)

function Ent:init(world, x, y, settings)
    Entity.init(self, world, x, y, settings)
    self:physicsRectangleInit(self.world.world, self.x, self.y, 'dynamic', 80, 40)
    self:timerInit()
    self:hittableInit(0.1)
    self:steerableInit(0, 0, 100, 1000)

    self.offset = Vector(0, -48)
    self.animations = {}
    self.animations['walk_left'] = newAnimation(visuals['ent_walk_left'], 132, 130, 0.2, 0)
    self.animations['walk_right'] = newAnimation(visuals['ent_walk_right'], 132, 130, 0.2, 0)
    self.animations['attack_left'] = newAnimation(visuals['ent_attack_left'], 132, 130, 0.23, 0)
    self.animations['attack_right'] = newAnimation(visuals['ent_attack_right'], 132, 130, 0.23, 0)

    self.hp = 3000 
    self.particle_colors = {{172, 106, 52}, {142, 88, 43}, {102, 63, 31}}
    self.attacking = false 
    self.skill = false
    self.skill_color = {128, 128, 128}
    self.timer:every(math.prandom(4, 6), function() self:act() end)
    self.timer:every(math.prandom(2, 4), function() self:act_attack() end)
    self.attack_colors_g = {{98, 111, 23}, {129, 146, 30}, {111, 126, 27}}
    self.attack_colors = {{128, 156, 35}, {140, 170, 50}, {166, 201, 66}} 

    self.can_attack = true 
    self.attacking = false
end

function Ent:update(dt)
    self.x, self.y = self.body:getPosition()
    self:timerUpdate(dt)
    self:steerableUpdate(dt)

    if self.moving then
        if self.direction == 'left' then
            self.animations['walk_left']:update(dt)
        else self.animations['walk_right']:update(dt) end
    else
        if self.direction == 'left' then
            self.animations['walk_left']:seek(1)
        else self.animations['walk_right']:seek(1) end
    end

    if self.attacking then
        if self.direction == 'left' then
            self.animations['attack_left']:update(dt)
        else self.animations['attack_right']:update(dt) end
    end
end

function Ent:act()
    local r = math.prandom(0, 2*math.pi)
    local d = math.prandom(50, 200)
    self.current_behavior = 'arrival'
    self.target = Vector(self.x + d*math.cos(r), self.y + d*math.sin(r))
end

function Ent:act_attack()
    local player_x, player_y = self.world.player.x, self.world.player.y
    local d = Vector.distance(Vector(player_x, player_y), Vector(self.x, self.y))
    local v = Vector(player_x - self.x, player_y - self.y)
    if d < 500 then self:attack(v:angle()) 
    elseif dice(0.5) then self:attack(v:angle()) end
end

function Ent:attack(r)
    if self.can_attack then
        self.attacking = true
        self.direction = angleToDirection(r)
        self.timer:after(0.3, function()
            self.animations['attack_left']:seek(1)
            self.animations['attack_right']:seek(1)
            self.attacking = false
            self.world:cameraShakeAdd(10, 0.6)
            for i = 1, 72 do
                local r = (i-1)*math.pi/36
                self.world:createToGroup('Projectile', 'Projectile',
                self.x + math.cos(r)*15, self.y + math.sin(r)*15,
                {ang_v = {2*math.pi, 4*math.pi}, r = r, v = math.random(100, 500), dmg = 100, range = math.prandom(2, 4), size = math.prandom(1, 3), 
                color = table.random(self.attack_colors_g), owner_id = self.id})
                self.can_attack = false
                self.timer:after(5, function() self.can_attack = true end)
            end
        end)
    end
end

function Ent:projectileHit(r, dmg)
    self:hittableHit()
    local color = table.random(self.particle_colors)
    local c1, c2 = {color[1], color[2], color[3], 255}, {color[1], color[2], color[3], 0}
    self.world:particleCreate('ProjHit', {position = {x = self.x, y = self.y},
                              colors = {c1[1], c1[2], c1[3], c1[4], c2[1], c2[2], c2[3], c2[4]}})
    self.hp = self.hp - dmg
    self.world:createToGroup('NumberPopText', 'NumberPopText', self.x, self.y,
                            {size = 'small', text = tostring(dmg), color = {240, 160, 160}})
    if self.hp <= 0 then
        self.world:hitFrameStopAdd(20)
        self.timer:after(0.02, function() 
            self.dead = true 
            for i = 1, 4 do
                local item_type = table.random(item_types)
                self.world:create('Item', self.x + math.random(-10, 10), self.y + math.random(-10, 10),
                                 {item_type = item_type, visual = visuals['items'][item_type]})
            end
        end)
    end
end

function Ent:draw()
    if debug_draw then self:physicsRectangleDraw() end
    love.graphics.setColorMode('combine')
    local sc = {self.skill_color[1], self.skill_color[2], self.skill_color[3]}
    local hc = {self.hit_color[1], self.hit_color[2], self.hit_color[3]}
    if self.hit then love.graphics.setColor((sc[1]+hc[1])/2, (sc[2]+hc[2])/2, (sc[3]+hc[3])/2, 255)
    else love.graphics.setColor(sc[1], sc[2], sc[3], 255) end

    local x, y = self.body:getPosition()
    x, y = x - 66 + self.offset.x, y - 65 + self.offset.y
    if not self.attacking then
        if self.direction == 'left' then
            self.animations['walk_left']:draw(x, y)
        else self.animations['walk_right']:draw(x, y) end
    elseif self.attacking then
        if self.direction == 'left' then
            self.animations['attack_left']:draw(x, y)
        else self.animations['attack_right']:draw(x, y) end
    end

    love.graphics.setColorMode('modulate')
    love.graphics.setColor(255, 255, 255)
end
