Entie = class('Entie', Entity)
Entie:include(PhysicsRectangle)
Entie:include(Timer)
Entie:include(HittableRed)
Entie:include(Steerable)

function Entie:init(world, x, y, settings)
    Entity.init(self, world, x, y, settings)
    self:physicsRectangleInit(self.world.world, self.x, self.y, 'dynamic', 32, 32)
    self:timerInit()
    self:hittableInit(0.2)
    self:steerableInit(0, 0, 200, 1000)

    self.offset = Vector(0, 0)
    self.animations = {}
    self.animations['walk'] = newAnimation(visuals['entie_walk'], 35, 34, 0.3, 0)
    self.can_attack = true
    self.body:setFixedRotation(true)

    self.particle_colors = {{172, 106, 52}, {142, 88, 43}, {102, 63, 31}}
    self.hp = 40
    self.timer:every(math.prandom(2, 4), function() self:act() end)
    self.timer:every(math.prandom(1, 3), function() self:act_attack() end)
end

function Entie:update(dt)
    self.x, self.y = self.body:getPosition()
    self:timerUpdate(dt)
    self:steerableUpdate(dt)
    self.animations['walk']:update(dt)
end

function Entie:act()
    local r = math.prandom(0, 2*math.pi)
    local d = math.prandom(50, 200)
    self.current_behavior = 'arrival'
    self.target = Vector(self.x + d*math.cos(r), self.y + d*math.sin(r))
end

function Entie:act_attack()
    if self.world.player.dead then return end
    local player_x, player_y = self.world.player.x, self.world.player.y
    local d = Vector.distance(Vector(player_x, player_y), Vector(self.x, self.y))
    local v = Vector(player_x - self.x, player_y - self.y)
    if d < 350 then
        self:attack(v:angle())
    end
end

function Entie:attack(r)
    if self.can_attack then
        self.can_attack = false
        self.world:createToGroup('Projectile', 'Projectile',
        self.x + math.cos(r)*15, self.y + math.sin(r)*15,
        {ang_v = {math.pi/2, 2*math.pi}, r = r + math.prandom(-math.pi/16, math.pi/16), v = math.prandom(100, 200), trail = 0.1,
        dmg = math.random(10, 20), range = 6, size = 1, color = table.random(self.particle_colors), owner_id = self.id})
        self.timer:after(4, function() self.can_attack = true end)
    end
end

function Entie:projectileHit(id, r, dmg)
    self:hittableHit()
    local color = table.random(self.particle_colors)
    local c1, c2 = {color[1], color[2], color[3], 255}, {color[1], color[2], color[3], 0}
    self.world:particleCreate('ProjHit', {position = {x = self.x, y = self.y},
                              colors = {c1[1], c1[2], c1[3], c1[4], c2[1], c2[2], c2[3], c2[4]}})
    self.hp = self.hp - dmg
    self.world:createToGroup('NumberPopText', 'NumberPopText', self.x, self.y,
                            {size = 'small', text = tostring(dmg), color = {240, 160, 160}})
    if self.hp <= 0 then 
        if id == nil then
            self.world:hitFrameStopAdd(4)
        end
        self.timer:after(0.02, function() self.dead = true end)
    end
end

function Entie:draw()
    if debug_draw then self:physicsRectangleDraw() end
    self:hittableDraw()

    local x, y = self.body:getPosition()
    x, y = x - self.w/2 + self.offset.x, y - self.h/2 + self.offset.y
    self.animations['walk']:draw(x, y)

    love.graphics.setColorMode('modulate')
    love.graphics.setColor(255, 255, 255)
end
