Monk = class('Monk', Entity)
Monk:include(PhysicsRectangle)
Monk:include(Timer)
Monk:include(HittableRed)
Monk:include(Steerable)
Monk:include(Speaker)

function Monk:init(world, x, y, settings)
    Entity.init(self, world, x, y, settings)
    self:physicsRectangleInit(self.world.world, self.x, self.y, 'dynamic', 26, 18)
    self:timerInit()
    self:hittableInit(0.2)
    self:steerableInit(0, 0, 100, 1000)

    self.offset = Vector(-2, -12)
    self.animations = {}
    self.animations['walk_left'] = newAnimation(visuals['monk_walk_left'], 36, 34, 0.2, 0)
    self.animations['walk_right'] = newAnimation(visuals['monk_walk_right'], 36, 34, 0.2, 0)
    self.animations['attack_left'] = newAnimation(visuals['monk_attack_left'], 64, 34, 0.2, 0)
    self.animations['attack_right'] = newAnimation(visuals['monk_attack_right'], 64, 34, 0.2, 0)

    self.timer:every(math.prandom(1, 3), function() self:act() end)
    self.timer:every(math.prandom(1, 2), function() self:act_attack() end)
    self.current_behavior = 'stop'

    self.hp = 50
    self.particle_colors = {{172, 52, 52}, {200, 64, 64}, {220, 92, 92}}
    self.arrival2_radius = math.random(200, 300)

    self.seek_texts = {
        "stop killing our children!", "i'll end you!", "murderer!!!",
        "get him!", "stop right there, criminal scum!"
    }

    self.attack_cooldown = 1
    self.can_attack = true
    self.attacking = false
    self.skill_1_colors = {{100, 160, 220}, {120, 160, 220}, {140, 180, 220}, {160, 180, 220}}
    self.skill_color = {128, 128, 128}
end

function Monk:update(dt)
    self.x, self.y = self.body:getPosition()
    self:timerUpdate(dt)
    self:steerableUpdate(dt)

    if self.moving then
        if self.direction == 'left' then
            self.animations['walk_left']:update(dt)
        else self.animations['walk_right']:update(dt) end
    else
        if self.direction == 'left' then
            self.animations['walk_left']:seek(1)
        else self.animations['walk_right']:seek(1) end
    end

    if self.attacking then
        if self.direction == 'left' then
            self.animations['attack_left']:update(dt)
        else self.animations['attack_right']:update(dt) end
    end


    if self.world.player.dead then 
        self.current_behavior = 'stop' 
        return
    end
    local player_x, player_y = self.world.player.x, self.world.player.y
    local d = Vector.distance(Vector(player_x, player_y), Vector(self.x, self.y))
    if d < 350 then
        if self.current_behavior ~= 'push' then
            self.current_behavior = 'arrival2'
            self.target = Vector(player_x, player_y)
        end
    end
end

function Monk:collisionSolid(solid, nx, ny)
    self.v.x = -self.v.x
    self.v.y = -self.v.y
end

function Monk:projectileHit(id, r, dmg)
    self:hittableHit()
    if not self.skill then self:steerablePush(150, r) end
    local color = table.random(self.particle_colors)
    local c1, c2 = {color[1], color[2], color[3], 255}, {color[1], color[2], color[3], 0}
    self.world:particleCreate('ProjHit', {position = {x = self.x, y = self.y},
                              colors = {c1[1], c1[2], c1[3], c1[4], c2[1], c2[2], c2[3], c2[4]}})
    self.hp = self.hp - dmg
    self.world:createToGroup('NumberPopText', 'NumberPopText', self.x, self.y,
                            {size = 'small', text = tostring(dmg), color = {240, 160, 160}})
    if self.hp <= 0 then
        if id == nil then
            self.world:hitFrameStopAdd(7)
        end
        self.timer:after(0.02, function() self.dead = true end)
    end
end

function Monk:act()
    if self.current_behavior ~= 'push' then
        local r = math.prandom(0, 2*math.pi)
        local d = math.prandom(50, 150)
        self.current_behavior = 'arrival'
        self.target = Vector(self.x + d*math.cos(r), self.y + d*math.sin(r))
    end
    if self.world.player.dead then return end
    if dice(0.2) then
        self.skill = true
        self.timer:tween(1, self, {skill_color = {108, 108, 240}}, 'in-out-cubic')
        self.timer:after(4, function()
            self.timer:tween(1, self, {skill_color = {128, 128, 128}}, 'in-out-cubic')
            self.timer:after(1, function() self.skill = false end)
        end)
    end
    if dice(0.1) then
        self:speak(table.random(self.seek_texts))
    end
end

function Monk:act_attack()
    if self.world.player.dead then return end
    local player_x, player_y = self.world.player.x, self.world.player.y
    local d = Vector.distance(Vector(player_x, player_y), Vector(self.x, self.y))
    local v = Vector(player_x - self.x, player_y - self.y)
    if d < 350 then
        self:attack(v:angle())
    end
end

function Monk:attack(r)
    if self.can_attack then
        self.attacking = true
        self.direction = angleToDirection(r)
        if not self.skill then
            self.world:createToGroup('Projectile', 'Projectile',
            self.x + math.cos(r)*15, self.y + math.sin(r)*15,
            {ang_v = {math.pi/2, 2*math.pi}, r = r + math.prandom(-math.pi/16, math.pi/16), v = math.prandom(400, 600),
            dmg = math.random(25, 75), range = 1, size = 1, color = table.random(self.skill_1_colors), owner_id = self.id})
            self.can_attack = false
            self.timer:after(0.4, function() self.attacking = false end)
            self.timer:after(self.attack_cooldown, function() self.can_attack = true end)
        else
            self.timer:every(0.05, function()
                self.world:createToGroup('Projectile', 'Projectile',
                self.x + math.cos(r)*15, self.y + math.sin(r)*15,
                {ang_v = {math.pi/2, 2*math.pi}, r = r + math.prandom(-math.pi/16, math.pi/16), v = math.prandom(400, 600),
                dmg = math.random(25, 75), range = 1, size = 1, color = table.random(self.skill_1_colors), owner_id = self.id})
            end, 10)
            self.can_attack = false
            self.timer:after(0.4, function() self.attacking = false end)
            self.timer:after(self.attack_cooldown, function() self.can_attack = true end)
        end
    end
end

function Monk:draw()
    if debug_draw then self:physicsRectangleDraw() end
    love.graphics.setColorMode('combine')
    local sc = {self.skill_color[1], self.skill_color[2], self.skill_color[3]}
    local hc = {self.hit_color[1], self.hit_color[2], self.hit_color[3]}
    if self.hit then love.graphics.setColor((sc[1]+hc[1])/2, (sc[2]+hc[2])/2, (sc[3]+hc[3])/2, 255)
    else love.graphics.setColor(sc[1], sc[2], sc[3], 255) end

    local x, y = self.body:getPosition()
    x, y = x - self.w/2 + self.offset.x, y - self.h/2 + self.offset.y
    if not self.attacking then
        if self.direction == 'left' then
            self.animations['walk_left']:draw(x, y)
        else self.animations['walk_right']:draw(x, y) end
    elseif self.attacking then
        if self.direction == 'left' then
            self.animations['attack_left']:draw(x - 28, y)
        else self.animations['attack_right']:draw(x, y) end
    end

    love.graphics.setColorMode('modulate')
    love.graphics.setColor(255, 255, 255)
end
