Player = class('Player', Entity)
Player:include(PhysicsRectangle)
Player:include(Timer)
Player:include(HittableRed)
Player:include(Movable)
Player:include(PlayerVisual)
Player:include(Attacker)

function Player:init(world, x, y, settings)
    Entity.init(self, world, x, y, settings)
    self:physicsRectangleInit(self.world.world, self.x, self.y, 'dynamic', 32, 12)
    self:timerInit()
    self:hittableInit(0.04)
    self:movableInit(200, 2000, 1)
    self:visualInit(Vector(0, -12))
    self:attackerInit()

    input:bind('left', {'a', 'left'})
    input:bind('right', {'d', 'right'})
    input:bind('up', {'w', 'up'})
    input:bind('down', {'s', 'down'})
    input:bind('shoot', {'lmb'})
    input:bind('skill_1', {'1'})
    input:bind('skill_2', {'2'})
    input:bind('skill_3', {'3'})
    input:bind('skill_4', {'4'})

    self.hp = 300
    self.hp_regen = 6
    self.mana = 100
    self.mana_regen = 2
    self.timer:every(1, function() 
        self.hp = self.hp + self.hp_regen
        if self.hp > 300 then self.hp = 300
        else
            self.world:createToGroup('NumberPopText', 'NumberPopText', self.x, self.y, 
                                    {size = 'small', text = tostring(self.hp_regen), color = {186, 240, 186}}) 
        end
        self.mana = self.mana + self.mana_regen
        if self.mana > 100 then self.mana = 100
        else 
            self.world:createToGroup('NumberPopText', 'NumberPopText', self.x, self.y, 
                                    {size = 'small', text = "+" .. tostring(self.mana_regen), color = {186, 186, 240}}) 
        end
    end)

    self.particle_colors = {{172, 52, 52}, {200, 64, 64}, {220, 92, 92}}
    self.n_rogues = {}
    self.n_enties = {}

    self.timer:every(0.3, function()
        if #self.n_rogues >= 1 then
            self:takeDamage(math.random(10, 50))
            for i = 1, #self.n_rogues-1 do
                self.timer:after(0.1, function() self:takeDamage(math.random(10, 50)) end)
            end
        end
    end)

    self.timer:every(0.2, function()
        if #self.n_enties >= 1 then
            self:takeDamage(math.random(10, 25))
            for i = 1, #self.n_enties-1 do
                self.timer:after(0.05, function() self:takeDamage(math.random(10, 25)) end)
            end
        end
    end)
end

function Player:takeDamage(dmg)
    self:hittableHit()
    self.world:cameraShakeAdd(5, 0.2)
    self.world:hitFrameStopAdd(3)
    local color = table.random(self.particle_colors)
    local c1, c2 = {color[1], color[2], color[3], 255}, {color[1], color[2], color[3], 0}
    self.world:particleCreate('ProjHit', {position = {x = self.x, y = self.y},
                              colors = {c1[1], c1[2], c1[3], c1[4], c2[1], c2[2], c2[3], c2[4]}})
    self.hp = self.hp - dmg
    self.world:createToGroup('NumberPopText', 'NumberPopText', self.x, self.y,
                            {size = 'small', text = tostring(dmg), color = {240, 160, 160}})
    if self.hp <= 0 then
        self.world:hitFrameStopAdd(20)
        self.n_rogues = {}
        self.n_enties = {}
        self.timer:after(0.02, function() self.dead = true end)
        timer:after(1, function() game.current_world.lost = true end)
    end
end

local rogueColliding = function(tbl, rogue)
    for _, t in ipairs(tbl) do
        if t.id == rogue.id then
            return true
        end
    end
    return false
end

local rogueRemove = function(tbl, rogue)
    for i, t in ipairs(tbl) do
        if t.id == rogue.id then
            table.remove(tbl, i)
            return
        end
    end
end

function Player:update(dt)
    self.x, self.y = self.body:getPosition()
    self:timerUpdate(dt)
    self:movableUpdate(dt)
    self:visualUpdate(dt)
    self:attackerUpdate(dt)
    for i = #self.n_rogues, 1, -1 do
        if self.n_rogues[i].dead then 
            rogueRemove(self.n_rogues, self.n_rogues[i])
        end
    end
    for i = #self.n_enties, 1, -1 do
        if self.n_enties[i].dead then
            rogueRemove(self.n_enties, self.n_enties[i])
        end
    end
end

function Player:draw()
    if debug_draw then self:physicsRectangleDraw() end
    self:hittableDraw()
    self:attackerDraw()
    self:visualDraw()
    love.graphics.setColorMode('modulate')
    love.graphics.setColor(255, 255, 255)
end

function Player:collisionRogueEnter(rogue, nx, ny)
    if not rogueColliding(self.n_rogues, rogue) then
        table.insert(self.n_rogues, rogue)
    end
end

function Player:collisionEntieEnter(entie, nx, ny)
    if not rogueColliding(self.n_enties, entie) then
        table.insert(self.n_enties, entie)
    end
end

function Player:collisionRogueExit(rogue, nx, ny)
    rogueRemove(self.n_rogues, rogue)
end

function Player:collisionEntieExit(entie, nx, ny)
    rogueRemove(self.n_enties, entie)
end

function Player:collisionProjectile(proj, nx, ny)
    if not proj.owner_id then return end
    self:takeDamage(proj.dmg)
end

function Player:collisionItem(item, nx, ny)
    if item.disabled then return end
    item.dead = true
    self.world:particleCreate('ItemGet', {position = {x = (self.x + item.x)/2, y = (self.y + item.y)/2},
                              colors = {255, 255, 255, 255, 255, 255, 255, 0}}) 
    self.world:createToGroup('InfoUpText', 'InfoUpText', self.x, self.y, {text = item.item_type})
    if item.item_type == 'damage up' then
        self.dmg_m = self.dmg_m + 0.3

    elseif item.item_type == 'attack speed up' then
        self.aspd_m = self.aspd_m - 0.1

    elseif item.item_type == 'mov speed up' then
        self.mov_m = self.mov_m + 0.2
        self.init_max_v = self.init_max_v*self.mov_m
        self.max_v = self.init_max_v

    elseif item.item_type == 'range up' then
        self.range_m = self.range_m + 0.25

    elseif item.item_type == 'mana regen up' then
        self.mana_regen = self.mana_regen + 1

    elseif item.item_type == 'hp regen up' then
        self.hp_regen = self.hp_regen + 3

    elseif item.item_type == 'mana cost down' then
        self.mana_m = self.mana_m - 0.2
    end
end
