Projectile = class('Projectile', Entity)
Projectile:include(PhysicsRectangle)
Projectile:include(Timer)

function Projectile:init(world, x, y, settings)
    Entity.init(self, world, x, y, settings)
    self:physicsRectangleInit(self.world.world, self.x, self.y, 'dynamic', 16, 16)
    self.body:setFixedRotation(false)
    self.body:setAngularVelocity(table.prandomBetween(settings.ang_v))
    self.body:setLinearVelocity(math.cos(settings.r)*settings.v,
                                math.sin(settings.r)*settings.v)
    self.r = settings.r
    self.dmg = settings.dmg or 5
    self.owner_id = settings.owner_id
    self.visual = settings.visual or projectile_b
    self:timerInit()
    self.scale = settings.size
    self.color = settings.color or {255, 255, 255}
    self.alpha = 255
    self.timer:tween(settings.range, self, {alpha = 0}, 'in-out-cubic')
    self.timer:after(settings.range*0.75, function() 
        if self.owner_id == self.world.ent_id then
            if dice(0.75) then
                self.world:create('Prop', self.x, self.y, {n = math.random(46, 54)})
            else self.world:create('Entie', self.x, self.y) end
        end
        self.dead = true 
        self.world:particleCreate('ProjectileDead', {position = {x = self.x, y = self.y},
                                  colors = {self.color[1], self.color[2], self.color[3], 255, self.color[1], self.color[2], self.color[3], 0}}) 
    end)
    self.timer:every(settings.trail or 0.035, function()
        self.world:createToGroup('Trail', 'Trail', self.x, self.y,
        {r = self.body:getAngle(), size = self.scale, color = self.color, visual = self.visual})
    end)
end

function Projectile:update(dt)
    self.x, self.y = self.body:getPosition()
    self:timerUpdate(dt)
end

function Projectile:draw()
    if debug_draw then self:physicsRectangleDraw() end
    love.graphics.setColor(self.color[1], self.color[2], self.color[3], self.alpha) 
    pushRotateScale(self.x, self.y, self.body:getAngle(), self.scale)
    local x, y = self.x - self.w/2, self.y - self.h/2 
    love.graphics.draw(self.visual, x + math.prandom(-4, 4), y + math.prandom(-4, 4))
    love.graphics.pop()
    love.graphics.setColor(255, 255, 255, 255)
end

function Projectile:collisionSolid(solid, nx, ny)
    self.dead = true
    self.world:particleCreate('Explosion1', {position = {x = self.x, y = self.y},
                              colors = {self.color[1], self.color[2], self.color[3], 255, self.color[1], self.color[2], self.color[3], 0}}) 
end

function Projectile:collisionProp(prop, nx, ny)
    if self.owner_id == self.world.ent_id then return end
    prop.dead = true
    if not self.owner_id then
        self.world:cameraShakeAdd(3, 0.2)
    end
    local color = table.random(prop.particle_colors)
    local c1, c2 = {color[1], color[2], color[3], 255}, {color[1], color[2], color[3], 0}
    self.world:particleCreate('EnemyDead', {position = {x = prop.x, y = prop.y}, 
                              colors = {c1[1], c1[2], c1[3], c1[4], c2[1], c2[2], c2[3], c2[4]}})
end

function Projectile:collisionBSolid(bsolid, nx, ny)
    self.dead = true
    self.world:particleCreate('Explosion1', {position = {x = self.x, y = self.y}, 
                              colors = {self.color[1], self.color[2], self.color[3], 255, self.color[1], self.color[2], self.color[3], 0}}) 
    if not self.owner_id then 
        self.world:cameraShakeAdd(5, 0.4)
        self.world:hitFrameStopAdd(3)
    end
    bsolid:projectileHit()
end

function Projectile:collisionBaby(baby, nx, ny)
    self.dead = true
    self.world:particleCreate('Explosion1', {position = {x = (self.x + baby.x)/2, y = (self.y + baby.y)/2},
                              colors = {self.color[1], self.color[2], self.color[3], 255, self.color[1], self.color[2], self.color[3], 0}}) 
    if not self.owner_id then
        self.world:cameraShakeAdd(3, 0.2)
        self.world:hitFrameStopAdd(3)
    end
    baby:projectileHit(self.r) 
end

function Projectile:collisionRogue(rogue, nx, ny)
    self.dead = true
    self.world:particleCreate('Explosion1', {position = {x = (self.x + rogue.x)/2, y = (self.y + rogue.y)/2},
                              colors = {self.color[1], self.color[2], self.color[3], 255, self.color[1], self.color[2], self.color[3], 0}}) 
    if not self.owner_id then
        self.world:cameraShakeAdd(3, 0.2)
        self.world:hitFrameStopAdd(3)
    end
    rogue:projectileHit(self.owner_id, self.r, self.dmg) 
end

function Projectile:collisionMonk(monk, nx, ny)
    if self.owner_id then 
        if self.owner_id == monk.id then return end
    end
    self.dead = true
    self.world:particleCreate('Explosion1', {position = {x = (self.x + monk.x)/2, y = (self.y + monk.y)/2},
                              colors = {self.color[1], self.color[2], self.color[3], 255, self.color[1], self.color[2], self.color[3], 0}}) 
    if not self.owner_id then
        self.world:cameraShakeAdd(3, 0.2)
        self.world:hitFrameStopAdd(3)
    end
    monk:projectileHit(self.owner_id, self.r, self.dmg) 
end

function Projectile:collisionEnt(ent, nx, ny)
    if self.owner_id then 
        if self.owner_id == ent.id then return end
    end
    self.dead = true
    self.world:particleCreate('Explosion1', {position = {x = (self.x + ent.x)/2, y = (self.y + ent.y)/2},
                              colors = {self.color[1], self.color[2], self.color[3], 255, self.color[1], self.color[2], self.color[3], 0}}) 
    if not self.owner_id then
        self.world:cameraShakeAdd(3, 0.2)
        self.world:hitFrameStopAdd(3)
    end
    ent:projectileHit(self.r, self.dmg) 
end

function Projectile:collisionEntie(entie, nx, ny)
    if self.owner_id then 
        if self.owner_id == self.world.ent_id then return end
        if self.owner_id == entie.id then return end
    end
    self.dead = true
    self.world:particleCreate('Explosion1', {position = {x = (self.x + entie.x)/2, y = (self.y + entie.y)/2},
                              colors = {self.color[1], self.color[2], self.color[3], 255, self.color[1], self.color[2], self.color[3], 0}}) 
    if not self.owner_id then
        self.world:cameraShakeAdd(3, 0.2)
        self.world:hitFrameStopAdd(3)
    end
    entie:projectileHit(self.owner_id, self.r, self.dmg) 
end
