-- Requires PhysicsRectangle
-- Requires Input (MOVE direction messages)
Movable = {
    movableInit = function(self, max_v, a, gravity_scale)
        self.init_max_v = max_v
        self.max_v = max_v 
        self.v = Vector(0, 0)
        self.init_a = a
        self.a = self.init_a
        self.damping = 0.8
        self.gravity_scale = gravity_scale
        self.direction = 'right'
        self.body:setGravityScale(gravity_scale)
        self.body:setFixedRotation(true)
        self.moving = false
        self.pushing = false
        self.push_force = 0
        self.push_angle = 0
        self.cant_move = false
    end,

    push = function(self, force, angle, trail)
        self.pushing = true
        self.push_angle = angle
        self.timer:tween(0.3, self, {push_force = force}, 'in-out-cubic')
        if trail then
            self.timer:every(0.035, function()
                if self.direction == 'left' then
                    self.world:create('Trail', self.x, self.y, {visual = aang_left, alpha = 144})
                else self.world:create('Trail', self.x, self.y, {visual = aang_right, alpha = 144}) end
            end, 10)
        end
        self.timer:after(0.5, function() 
            self.pushing = false 
            self.push_force = 0
            self.push_angle = 0
        end)
    end,
    
    movableUpdate = function(self, dt)
        self.moving = false
        if self.pushing then 
            self.v = Vector(self.push_force*math.cos(self.push_angle), self.push_force*math.sin(self.push_angle))
            self.body:setLinearVelocity(self.v.x, self.v.y)
            return
        end
        if self.cant_move then return end

        if input:down('up') then
            self.v.y = math.max(self.v.y - self.a*dt, -self.max_v)
            local vx, vy = self.body:getLinearVelocity()
            self.body:setLinearVelocity(vx, self.v.y)
            self.moving = true
        end

        if input:down('down') then
            self.v.y = math.min(self.v.y + self.a*dt, self.max_v)
            local vx, vy = self.body:getLinearVelocity()
            self.body:setLinearVelocity(vx, self.v.y)
            self.moving = true
        end

        if input:down('left') then
            if input:down('up') then
                self.v.x = math.max(self.v.x - self.a*dt, math.cos(3*math.pi/4)*self.max_v)
                self.v.y = math.max(self.v.y - self.a*dt, -math.sin(3*math.pi/4)*self.max_v)
                self.body:setLinearVelocity(self.v.x, self.v.y)
            elseif input:down('down') then
                self.v.x = math.max(self.v.x - self.a*dt, math.cos(-3*math.pi/4)*self.max_v)
                self.v.y = math.min(self.v.y + self.a*dt, -math.sin(-3*math.pi/4)*self.max_v)
                self.body:setLinearVelocity(self.v.x, self.v.y)
            else
                self.v.x = math.max(self.v.x - self.a*dt, -self.max_v)
                local vx, vy = self.body:getLinearVelocity()
                self.body:setLinearVelocity(self.v.x, vy)
            end
            if not self.attacking then self.direction = 'left' end
            self.moving = true
        end

        if input:down('right') then
            if input:down('up') then
                self.v.x = math.min(self.v.x + self.a*dt, math.cos(math.pi/4)*self.max_v)
                self.v.y = math.max(self.v.y - self.a*dt, -math.sin(math.pi/4)*self.max_v)
                self.body:setLinearVelocity(self.v.x, self.v.y)
            elseif input:down('down') then
                self.v.x = math.min(self.v.x + self.a*dt, math.cos(-math.pi/4)*self.max_v)
                self.v.y = math.min(self.v.y + self.a*dt, -math.sin(-math.pi/4)*self.max_v)
                self.body:setLinearVelocity(self.v.x, self.v.y)
            else
                self.v.x = math.min(self.v.x + self.a*dt, self.max_v)
                local vx, vy = self.body:getLinearVelocity()
                self.body:setLinearVelocity(self.v.x, vy)
            end
            if not self.attacking then self.direction = 'right' end
            self.moving = true
        end

        if not input:down('up') and not input:down('down') then
            self.v.y = self.v.y*self.damping
            if math.abs(self.v.y) < 24 then self.v.y = 0 end
            local vx, vy = self.body:getLinearVelocity()
            self.body:setLinearVelocity(vx, self.v.y)
        end

        if not input:down('left') and not input:down('right') then
            self.v.x = self.v.x*self.damping
            if math.abs(self.v.x) < 24 then self.v.x = 0 end
            local vx, vy = self.body:getLinearVelocity()
            self.body:setLinearVelocity(self.v.x, vy)
        end

        self.body:setGravityScale(self.gravity_scale)
    end
}
