-- Requires PhysicsRectangle
Steerable = {
    steerableInit = function(self, angle, v, max_v, max_f)
        self.current_behavior = 'idle'
        self.angle = angle or 0
        self.direction = angleToDirection(angle)
        self.v = Vector(math.cos(angle)*v, math.sin(angle)*v)
        self.init_max_v = max_v or 0
        self.max_v = max_v or 0
        self.max_f = max_f or 0
        self.steering = Vector(0, 0)
        self.steering_force = Vector(0, 0)
        self.target = Vector(0, 0)
        self.damping = 0.92
        self.stopping = false
        self.arrival_radius = 25
        self.arrival2_radius = 100
        self.flee_radius = 200
        self.moving = false
    end,

    steerableUpdate = function(self, dt)
        -- Only stop if self:steerableStop sets it to true
        -- otherwise reset every frame so the entity can move
        self.stopping = false

        -- Calculate self.steering based on the current behavior
        if self.current_behavior == 'seek' then
            self:steerableSeek(self.target)
        elseif self.current_behavior == 'arrival' then
            self:steerableArrival(self.target)
        elseif self.current_behavior == 'arrival2' then
            self:steerableArrival2(self.target)
        elseif self.current_behavior == 'flee' then
            self:steerableFlee(self.target)
        elseif self.current_behavior == 'stop' then
            self:steerableStop()
        else

        end

        if self.current_behavior ~= 'stop' then self.moving = true
        else self.moving = false end

        -- Steer
        if self.current_behavior ~= 'push' and self.current_behavior ~= 'stop' then
            self.steering_force = self.steering:min(self.max_f)
            if self.current_behavior == 'flee' then
                self.v = (self.v + self.steering_force*dt):min(20*self.max_v)
            else 
                self.v = (self.v + self.steering_force*dt):min(self.max_v)
            end
        end
        if self.current_behavior == 'stop' then
            self.steering_force = self.steering:min(self.max_f)
            self.v = (self.v + self.steering_force*dt):min(self.max_v)
        end
        self.body:setLinearVelocity(self.v.x, self.v.y)
        self.angle = self.v:angle()
        self.direction = angleToDirection(self.angle)

        -- Stop movement completely when close to 0
        if self.stopping then
            local vx, vy = math.round(self.v.x, 1), math.round(self.v.y, 1)
            if math.abs(vx) < 5 and math.abs(vy) < 5 then
                self.v.x, self.v.y = 0, 0
            end
        end

        if self.current_behavior == 'flee' then
            local vx, vy = math.round(self.v.x, 1), math.round(self.v.y, 1)
            if math.abs(vx) < 5 and math.abs(vy) < 5 then
                self.v.x, self.v.y = 0, 0
                self.current_behavior = 'stop'
            end
        end
    end,

    steerablePush = function(self, force, r)
        self.current_behavior = 'push'
        self.timer:after(0.5, function() self.current_behavior = 'stop' end)
        local vx, vy = force*math.cos(r), force*math.sin(r)
        self.v.x, self.v.y = vx, vy
    end,

    steerableStop = function(self)
        self.stopping = true 
        self.steering = Vector(0, 0) 
        self.v = self.v*self.damping
    end,

    steerableSeek = function(self, target)
        local position = Vector(self.body:getPosition())
        local desired_velocity = (target - position):normalized()*self.max_v
        local velocity = Vector(self.body:getLinearVelocity())
        self.steering = desired_velocity - velocity
    end,

    steerableFlee = function(self, target)
        local position = Vector(self.body:getPosition())
        local desired_velocity = (position - target)
        local distance = desired_velocity:len()
        if distance < self.flee_radius then
            desired_velocity = desired_velocity:normalized()*20*self.max_v*(1-(distance/self.flee_radius))
        else
            desired_velocity = Vector(0, 0)
        end
        local velocity = Vector(self.body:getLinearVelocity())
        self.steering = desired_velocity - velocity 
    end,

    steerableArrival = function(self, target)
        local position = Vector(self.body:getPosition())
        local desired_velocity = (target - position)
        local distance = desired_velocity:len()
        if distance < self.arrival_radius then
            self.current_behavior = 'stop'
            desired_velocity = desired_velocity:normalized()*self.max_v*(distance/self.arrival_radius)
        else
            desired_velocity = desired_velocity:normalized()*self.max_v
        end
        local velocity = Vector(self.body:getLinearVelocity())
        self.steering = desired_velocity - velocity
    end,

    steerableArrival2 = function(self, target)
        local position = Vector(self.body:getPosition())
        local desired_velocity = (target - position)
        local distance = desired_velocity:len()
        if distance < self.arrival2_radius then
            self.current_behavior = 'stop'
            desired_velocity = Vector(0, 0)
        else
            desired_velocity = desired_velocity:normalized()*self.max_v
        end
        local velocity = Vector(self.body:getLinearVelocity())
        self.steering = desired_velocity - velocity
    end,

    steerableDraw = function(self)
        if self.target then
            love.graphics.line(self.x, self.y, self.target.x, self.target.y)
            love.graphics.circle('line', self.target.x, self.target.y, self.arrival_radius)
        end

    end
}
