Group = class('Group')

function Group:init(name)
    self.name = name
    self.entities = {}
end

function Group:update(dt)
    for _, entity in ipairs(self.entities) do entity:update(dt) end
end

function Group:draw()
    for _, entity in ipairs(self.entities) do entity:draw(dt) end
end

function Group:add(entity)
    table.insert(self.entities, entity)
end

function Group:remove(id)
    table.remove(self.entities, findIndexByID(self.entities, id))
end

function Group:removePostWorldStep()
    for i = #self.entities, 1, -1 do
        if self.entities[i].dead then
            if self.entities[i].class:includes(Timer) then self.entities[i]:timerDestroy() end
            if self.entities[i].class:includes(PhysicsRectangle) then
                self.entities[i].fixture:setUserData(nil)
                self.entities[i].sensor:setUserData(nil)
                self.entities[i].body:destroy()
            end
            self.entities[i].world = nil
            self:remove(self.entities[i].id)
        end
    end
end

function Group:destroy()
    self.entities = nil
end

function Group:getEntities()
    return self.entities
end

function Group:apply(action)
    for _, entity in ipairs(self.entities) do action(entity) end
end

function Group:call(action_name, ...)
    for _, entity in ipairs(self.entities) do
        if entity[action_name] then entity[action_name](...) end
    end
end
