local room = struct('width', 'height', 'layout_data', 'offset')

Map = {
    mapInit = function(self, name)
        self.rooms = {}
        self.at_least_one_baby = Vector(math.random(2, 3), math.random(2, 3))
        self:createMapBorders()
        self:loadRooms()
    end,

    createMapBorders = function(self)
        self:create('Solid', -32, 40*32, {w = 64, h = 80*32, visual = tiles_data['environment_32'][1]})
        self:create('Solid', 80*32+32, 40*32, {w = 64, h = 80*32, visual = tiles_data['environment_32'][1]})
        self:create('Solid', 40*32, -32, {w = 84*32, h = 64, visual = tiles_data['environment_32'][1]})
        self:create('Solid', 40*32, 80*32+32, {w = 84*32, h = 64, visual = tiles_data['environment_32'][1]})
    end,

    loadRooms = function(self)
        for i = 1, 4 do
            for j = 1, 4 do
                self:loadRoom(i, j, (i-1)*20*32, (j-1)*20*32)
            end
        end
        self:create('Ent', 2*20*32, 2*20*32)
    end,

    loadRoom = function(self, i, j, x, y)
        local room_name = nil
        if i == 1 and j == 1 then room_name = table.random({'n5', 'n6'})
        elseif i == self.at_least_one_baby.x and j == self.at_least_one_baby.y then
            room_name = table.random(baby_pieces)
        else room_name = table.random(all_pieces) end

        local data = require('resources/' .. room_name)
        local current_room = room(data.layers[1].width, data.layers[1].height, {}, Vector(x, y))
        for _, layer in ipairs(data.layers) do
            if layer.type == 'tilelayer' then
                for i = 1, current_room.height do
                    current_room.layout_data[i] = {}
                end
                local i, j = 1, 1
                for _, v in ipairs(layer.data) do
                    current_room.layout_data[i][j] = v
                    j = j + 1
                    if j > current_room.width then j = 1; i = i + 1 end
                end
                table.insert(self.rooms, current_room)

            elseif layer.type == 'objectgroup' then
                for _, object in ipairs(layer.objects) do
                    if object.type == 'Solid' then
                        self:create('Solid', 
                                    object.x+object.width/2+current_room.offset.x, 
                                    object.y+object.height/2+current_room.offset.y, 
                                    {w = object.width, h = object.height, visual = tiles_data['environment_32'][17]})

                    elseif object.type == 'Prop' then
                        self:create('Prop',
                                    object.x+object.width/2+current_room.offset.x,
                                    object.y+object.height/2+current_room.offset.y,
                                    {w = object.width, h = object.height, tile_data = 'props_trees', n = math.random(46, 54)})

                    elseif object.type == 'BSolid' then
                        self:create('BSolid',
                                    object.x+object.width/2+current_room.offset.x,
                                    object.y+object.height/2+current_room.offset.y,
                                    {w = object.width, h = object.height, visual = tiles_data['environment_32'][20]})

                    elseif object.type == 'Baby' then
                        self:create('Baby', object.x+object.width/2+current_room.offset.x, object.y+object.height/2+current_room.offset.y)
                        self.game.n_babies = self.game.n_babies + 1

                    elseif object.type == 'Rogue' then
                        self:create('Rogue', object.x+object.width/2+current_room.offset.x, object.y+object.height/2+current_room.offset.y)

                    elseif object.type == 'Monk' then
                        self:create('Monk', object.x+object.width/2+current_room.offset.x, object.y+object.height/2+current_room.offset.y)

                    elseif object.type == 'RogueMonk' then
                        if dice(0.5) then
                            self:create('Rogue', object.x+object.width/2+current_room.offset.x, object.y+object.height/2+current_room.offset.y)
                        else
                            self:create('Monk', object.x+object.width/2+current_room.offset.x, object.y+object.height/2+current_room.offset.y)
                        end

                    elseif object.type == 'Item' then
                        local item_type = table.random(item_types)
                        self:create('Item', 
                                    object.x+object.width/2+current_room.offset.x, 
                                    object.y+object.height/2+current_room.offset.y,
                                    {item_type = item_type, visual = visuals['items'][item_type]})
                    end
                end
            end
        end
    end,

    mapDraw = function(self)
        for _, room in ipairs(self.rooms) do
            love.graphics.draw(grass, room.offset.x, room.offset.y)
            for i = 1, room.height do
                for j = 1, room.width do
                    if room.layout_data[i][j] ~= 0 then
                        love.graphics.draw(tiles_data['environment_32'][room.layout_data[i][j]],
                                          (j-1)*32+room.offset.x, (i-1)*32+room.offset.y)
                    end
                end
            end
        end
    end
}
