function love.load()
    love.graphics.setDefaultImageFilter('nearest', 'nearest')
    love.mouse.setVisible(false)

    class = require 'libraries/middleclass/middleclass'
    struct = require 'libraries/struct/struct'
    beholder = require 'libraries/beholder/beholder'
    Camera = require 'libraries/hump/camera'
    require 'libraries/Vector'
    GTimer = require 'libraries/hump/timer'
    require 'libraries/anal/AnAL'
    require 'libraries/TEsound'
    require 'utils'

    -- Game
    require 'game/Game'
    require 'game/Input'
    require 'game/Sound'

    -- Data
    require 'game/data/collision'
    require 'game/data/visual'
    require 'game/data/room'

    -- Mixins
    require 'game/mixins/PhysicsRectangle'
    require 'game/mixins/Timer'
    require 'game/mixins/HittableRed'
    require 'game/mixins/Movable'
    require 'game/mixins/PlayerVisual'
    require 'game/mixins/Attacker'
    require 'game/mixins/Steerable'
    require 'game/mixins/Speaker'
    
    -- Entities
    require 'game/entities/Entity'
    require 'game/entities/Player'
    require 'game/entities/Solid'
    require 'game/entities/Projectile'
    require 'game/entities/Trail'
    require 'game/entities/Prop'
    require 'game/entities/BSolid'
    require 'game/entities/NumberPopText'
    require 'game/entities/Baby'
    require 'game/entities/Rogue'
    require 'game/entities/Monk'
    require 'game/entities/Item'
    require 'game/entities/InfoUpText'
    require 'game/entities/Ent'
    require 'game/entities/Entie'

    uid = 0
    debug_draw = false
    game_width = love.graphics.getWidth()
    game_height = love.graphics.getHeight()
    timer = GTimer.new()
    game = Game()
    input = Input()
    sound = Sound()
    muted = false
end

function love.update(dt)
    timer:update(dt)
    game:update(dt)
    input:update(dt)
    sound:update(dt)
end

function love.draw()
    game:draw()
    -- love.graphics.setFont(tutorial_text)
    -- love.graphics.print(tostring(math.round(collectgarbage("count"), 0) .. 'Kb'), 10, 620)
    love.graphics.print(tostring(love.timer.getFPS()), 10, 600)
end

function love.mousepressed(x, y, button)
    input:mousepressed(x, y, button)
    game:mousepressed(x, y, button)
end

function love.mousereleased(x, y, button)
    input:mousereleased(x, y, button)
    game:mousereleased(x, y, button)
end

function love.keypressed(key)
    input:keypressed(key)
    game:keypressed(key)
    if key == 'c' then collectgarbage("collect") end
    if key == 'f1' then debug_draw = not debug_draw end
    if key == 'm' then
        muted = not muted
        if muted then TEsound.volume("music", 0)
        else TEsound.volume("music", 1) end
    end
end

function love.keyreleased(key)
    input:keyreleased(key)
    game:keyreleased(key)
end

-- 0.8
function love.run()
    math.randomseed(os.time())
    math.random(); math.random(); math.random();

    if love.load then love.load(arg) end

    local t = 0
    local dt = 0
    local fixed_dt = 1/60 
    local accumulator = 0

    -- Main loop time
    while true do
        -- Process events
        if love.event then
            love.event.pump()
            for e, a, b, c, d in love.event.poll() do
                if e == "quit" then
                    if not love.quit or not love.quit() then
                        if love.audio then
                            love.audio.stop()
                        end
                        return
                    end
                end
                love.handlers[e](a, b, c, d)
            end
        end

        -- Update dt, as we'll be passing it to update
        if love.timer then
            love.timer.step()
            dt = love.timer.getDelta()
        end

        accumulator = accumulator + dt

        while accumulator >= fixed_dt do
            if love.update then love.update(fixed_dt) end
            accumulator = accumulator - fixed_dt
            t = t + fixed_dt
        end

        if love.graphics then
            love.graphics.clear()
            if love.draw then love.draw() end
        end

        if love.timer then love.timer.sleep(0.001) end
        if love.graphics then love.graphics.present() end
    end
end
